local mod = RW:Boss("Ascendant Council", "The Bastion of Twilight")
local L
local LBB

mod.BossIcon = [[Interface\ICONS\Achievement_Dungeon_Bastion of Twilight_TwilightAscendantCouncil]]

---------------------------
-- Variables

--[23:58] Ignacious: 43686
--[23:59] Feludius: 43687
--[23:59] Terrastra: 43689
--[23:59] Arion: 43688

-- Elementium Monstrosity: 43735

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["Phase 2"] = "We will handle them!"
		L["Phase 3"] = "BEHOLD YOUR DOOM!"
	
		L["Interupt!"] = true
		L["Warning when interupt is possible"] = true
		
		L["%s Missing!"] = true
		L["Warning when needed debuff is missing"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	LBB = RW.LBB
	
	self:SetTrigger(43686, 43687, 43689, 43688, 43735)
	self:SetWin()
	
	self:AddToCategory(LBB["Feludius"] or "Feludius", 92508, 82762, 92307)
	self:AddToCategory(LBB["Ignacious"] or "Ignacious", 82631)
	self:AddToCategory(LBB["Terrastra"] or "Terrastra", 83565, 83718, 92067)
	self:AddToCategory(LBB["Arion"] or "Arion", 83099, 83067, 92075)
	
	-- Feludius
		-- Glaciate
		self:RegisterAlert("tGlaciateCast", self:Timer(3, 92508))
		self:RegisterAlert("tGlaciateCD", self:CDTimer(30, 92508))
		self:RegisterAlert("wGlaciate", self:RunawayWarning(92508))		
		-- Waterlogged
		self:RegisterAlert("wLogged", self:OnYouWarning(82762))
		
		-- Frost Beacon
		self:RegisterAlert("wBeacon", self:RunawayWarning(92307))
		self:RegisterAlert("iBeacon", self:Icon("TRIANGLE", 92307, false))
	
	-- Ignacious
		-- Aegis of Flame
		self:RegisterAlert("tAegisCD", self:CDTimer(61, 82631))
		self:RegisterAlert("wAegis", self:CastWarning(82631))
		self:RegisterAlert("wInterupt", self:CustomWarning(L["Interupt!"], L["Warning when interupt is possible"], 6552, 82631))
	
	-- Terrastra
		-- Quake
		self:RegisterAlert("wQuake", self:CastWarning(83565))
		self:RegisterAlert("tQuakeCast", self:Timer(3, 83565))
		self:RegisterAlert("tQuakeCD", self:CDTimer(50, 83565))
		
		-- Hardened Skin
		self:RegisterAlert("tSkinCD", self:CDTimer(60, 83718))
		self:RegisterAlert("tSkinDur", self:DurTimer(30, 83718))
		self:RegisterAlert("wSkin", self:CastWarning(83718))
		
		-- Static Overload
		self:RegisterAlert("wOverload", self:OnYouWarning(92067))
		self:RegisterAlert("iOverload", self:Icon("CROSS", 92067, false))
	
	-- Arion
		-- Lightning Rob
		self:RegisterAlert("wRod", self:RunawayWarning(83099))
		self:RegisterAlert("mRod", self:TargetMessage(83099))
		self:RegisterAlert("iRod", self:Icon("SKULL", 83099, false, 15, "DIAMOND", "CROSS"))

		-- Thundershock
		self:RegisterAlert("wShock", self:CastWarning(83067))
		self:RegisterAlert("tShockCast", self:Timer(3, 83067))
		self:RegisterAlert("tShockCD", self:CDTimer(50, 83067))		
		
		-- Gravity Core
		self:RegisterAlert("wCore", self:OnYouWarning(92075))
		self:RegisterAlert("iCore", self:Icon("SQUARE", 92075, false))
		
		
		
	self:RegisterAlert("wMissingDebuff", self:CustomWarning(L["%s Missing!"], L["Warning when needed debuff is missing"], nil, L["Warning when needed debuff is missing"]))
	self:RegisterAlert("mPhase", self:PhaseMessage())
end

---------------------------
-- Locals
local AegisSpell = GetSpellInfo(82631)
local SkinSpell = GetSpellInfo(83718)
local SwirlingWindsSpell = GetSpellInfo(83500)
local GroundedSpell = GetSpellInfo(83581)
local RodSpell = GetSpellInfo(83099)

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("Glaciate", 92508, 82746, 92506, 92507)	
	self:Track("Logged", 82762)
	self:Track("Beacon", 92307)
	
	self:Track("Aegis", 82631, 92512, 92513, 92514)
	
	self:Track("Quake", 83565, 92546, 92545, 92544)
	self:Track("Skin", 83718, 92541, 92542, 92543)
	self:Track("Overload", 92067)
	
	self:Track("Rod", 83099)
	self:Track("Shock", 83067, 92469, 92470, 92471)
	self:Track("Core", 92075)
	
	
	self:YellTrack("Phase2", L["Phase 2"])
	self:YellTrack("Phase3", L["Phase 3"])
	
	self:Call("tAegisCD", "Start", 30)
	self:Call("tGlaciateCD", "Start", 30)
end

function mod:OnEnd()
	RW.Callbacks:Fire("MPBExternalHide", "IgnaciousShieldHP")
	RW.Callbacks:Fire("MPBExternalHide", "TerrastraShieldHP")	
end

-- Feludius
do
	function mod:Glaciate(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("tGlaciateCast", "Start")
			self:Call("tGlaciateCD", "Start")
			self:Call("wGlaciate", "Show")
		end
	end

	function mod:Logged(event, args)
		if event == "SPELL_AURA_APPLIED" then
			if args:IsPlayer() then
				self:Call("wLogged", "Show")
			end
		end
	end
	
	function mod:Beacon(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("iBeacon", "Show", args.dName)
			if args:IsPlayer() then
				self:Call("wBeacon", "Show")
			end
		
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iBeacon", "Hide", args.dName)
		end
	end
end

-- Ignacious
do
	local showShieldHealthBar, hideShieldHealthBar
	do
		local frame = CreateFrame("Frame")
		local absorbRemaining = 0
		local maxAbsorb = 0
		local function getShieldHP()
			return absorbRemaining, maxAbsorb, AegisSpell
		end
		frame:SetScript("OnEvent", function(self, event, timestamp, subEvent, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
			if shieldedMob == destGUID then
				local absorbed
				if subEvent == "SPELL_MISSED" or subEvent == "SPELL_PERIODIC_MISSED" then
					absorbed = select(5, ...)
				elseif subEvent == "SWING_MISSED" then
					absorbed = select(2, ...)
				elseif subEvent == "RANGE_MISSED" then
					absorbed = select(5, ...)
				end
				if absorbed then
					absorbRemaining = absorbRemaining - absorbed
				end
			end
		end)
		
		function showShieldHealthBar()
			absorbRemaining = 	GetInstanceDifficulty() == 4 and 1500000 or
								GetInstanceDifficulty() == 3 and 700000 or
								GetInstanceDifficulty() == 2 and 1000000 or
								GetInstanceDifficulty() == 1 and 500000 or 0
			maxAbsorb = absorbRemaining
			RW.Callbacks:Fire("MPBExternalHide", "IgnaciousShieldHP")
			RW.Callbacks:Fire("MPBExternal", "IgnaciousShieldHP", getShieldHP, nil, nil, select(3, GetSpellInfo(82631)))
			mod:ScheduleTimer(hideShieldHealthBar, 30)
			frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
		
		function hideShieldHealthBar()
			RW.Callbacks:Fire("MPBExternalHide", "IgnaciousShieldHP")
			frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
	end

	function mod:Aegis(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("tAegisCD", "Start")
			self:Call("wAegis", "Show")
			showShieldHealthBar()
		
		elseif event == "SPELL_AURA_REMOVED" then
			hideShieldHealthBar()
		end
	end
end

-- Terrastra
do
	local showShieldHealthBar, hideShieldHealthBar
	do
		local frame = CreateFrame("Frame")
		local absorbRemaining = 0
		local maxAbsorb = 0
		local function getShieldHP()
			return absorbRemaining, maxAbsorb, SkinSpell
		end
		frame:SetScript("OnEvent", function(self, event, timestamp, subEvent, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
			if shieldedMob == destGUID then
				local absorbed
				if subEvent == "SPELL_MISSED" or subEvent == "SPELL_PERIODIC_MISSED" then
					absorbed = select(5, ...)
				elseif subEvent == "SWING_MISSED" then
					absorbed = select(2, ...)
				elseif subEvent == "RANGE_MISSED" then
					absorbed = select(5, ...)
				end
				if absorbed then
					absorbRemaining = absorbRemaining - absorbed
				end
			end
		end)
		
		function showShieldHealthBar()
			absorbRemaining = 	GetInstanceDifficulty() == 4 and 2100000 or
								GetInstanceDifficulty() == 3 and 650000 or
								GetInstanceDifficulty() == 2 and 1650000 or
								GetInstanceDifficulty() == 1 and 500000 or 0
			maxAbsorb = absorbRemaining
			RW.Callbacks:Fire("MPBExternalHide", "TerrastraShieldHP")
			RW.Callbacks:Fire("MPBExternal", "TerrastraShieldHP", getShieldHP, nil, nil, select(3, GetSpellInfo(83718)))
			mod:ScheduleTimer(hideShieldHealthBar, 30)
			frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
		
		function hideShieldHealthBar()
			RW.Callbacks:Fire("MPBExternalHide", "TerrastraShieldHP")
			frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
	end
	
	function mod:Quake(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("wQuake", "Show")
			self:Call("tQuakeCast", "Start")
			self:Call("tQuakeCD", "Start")
			
			local name = UnitDebuff("player", SwirlingWindsSpell)
			if not name and not UnitIsDeadOrGhost("player") then
				self:Call("wMissingDebuff", "Show", SwirlingWindsSpell)
			end
		end
	end
	
	function mod:Skin(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("wSkin", "Show")
			self:Call("tSkinCD", "Start")
			self:Call("tSkinDur", "Start")
			showShieldHealthBar()
			
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("tSkinDur", "Stop")
			hideShieldHealthBar()
		end
	end
	
	function mod:Overload(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("iOverload", "Show", args.dName)
			if args:IsPlayer() then
				self:Call("wOverload", "Show")
			end
		
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iOverload", "Hide", args.dName)
		end
	end
end

-- Arion
do
	function mod:Rod(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("iRod", "Show", args.dName)
			
			self:Call("mRod", "AddDelayTarget", args.dName)
			self:Call("mRod", "DelayedShow", 0.2)
			
			if args:IsPlayer() then
				self:Call("wRod", "Show")
				SendChatMessage(RodSpell.. " on "..self.pName.."!", "SAY")
			end
		
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iRod", "Hide", args.dName)
		end
	end
	
	function mod:Shock(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("wShock", "Show")
			self:Call("tShockCast", "Start")
			self:Call("tShockCD", "Start")
			
			local name = UnitDebuff("player", GroundedSpell)
			if not name and not UnitIsDeadOrGhost("player") then
				self:Call("wMissingDebuff", "Show", GroundedSpell)
			end
		end
	end
	
	function mod:Core(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("iCore", "Show", args.dName)
			if args:IsPlayer() then
				self:Call("wCore", "Show")
			end
		
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iCore", "Hide", args.dName)
		end
	end
end

function mod:Phase2(event, msg, name)
	--RW.Callbacks:Fire("MPBPowerHideByID", 43686)
	--RW.Callbacks:Fire("MPBPowerHideByID", 43687)
	self:Call("mPhase", "Show", 2)
end

function mod:Phase3(event, msg, name)
	--RW.Callbacks:Fire("MPBPowerHideByID", 43689)
	--RW.Callbacks:Fire("MPBPowerHideByID", 43688)
	self:Call("mPhase", "Show", 3)
end
